/*****************************************************************************
 *
 * Interrupt Demo for PIC24.
 *
 *****************************************************************************
 * FileName:        Lab3.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Vidyadhar		  30/June/06	     ...	
 * Brant Ivey			7/3/07		Updated for v4.0 of class
 * Brant Ivey		   10/19/07		Updated for v4.2 of class
 * Mark Reynolds		10/10/10	Updated for v4.3 addewd LCD to see state and slowed LEDs
 *****************************************************************************/

#include <p24FJ128GA010.h>

_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & ICS_PGx2 & FWDTEN_OFF)
_CONFIG2(IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_ON &  FNOSC_FRC & POSCMOD_NONE)

#define S3 PORTDbits.RD6
#define S4 PORTDbits.RD13
#define S5 PORTAbits.RA7
#define S6 PORTDbits.RD7
// Function Prototypes
void Delay(int ticks);
void ReadSwitch();
void TMR3Init();
void TMR5Init();
void IOInit();
char Sw, MyString[]="Case = 0";
int i;
/**********************************************************************************/

int main ()
{
	IOInit();
    TMR3Init();             
    TMR5Init();            
	LCDInit();
	while(1)
	{
		ReadSwitch();
		Delay(0xFFFF); //slow down loop
	
		IEC0bits.T3IE = 0;	// Disable Timer 3 interrupt to allow priority change
		IEC1bits.T5IE = 0;	// Disable Timer 5 interrupt to allow priority change
		switch (Sw)
		{
			case 0:                         //Disable LEDs while no button is pressed
			IPC2bits.T3IP = 0;			
			IPC7bits.T5IP = 0;
			LCDClear();
			LCDHome();
			i=0;
			while (MyString[i]!=0) LCDPut(MyString[i++]);			
			break;				


			case 1:                         //S3 Pressed
			SRbits.IPL = 0;	 // CPU priority set to 0

	
			//##########################################
			//  STEP 1:
			//	set Timer 3 interrupt priority level higher than Timer 5
			//##########################################
			
		              // ###
			          // ###
			LCDClear();
			LCDHome();
			i=0;
			while (MyString[i]!=0) LCDPut(MyString[i++]);			

			break;				


			case 2:                         //S6 Pressed
			SRbits.IPL = 0;	   // CPU priority set to 0

			//##########################################
			//  STEP 2:
			//  set Timer 5 interrupt priority level higher than Timer 3
			//##########################################
			
			          // ###
			          // ###
			LCDClear();
			LCDHome();
			i=0;
			while (MyString[i]!=0) LCDPut(MyString[i++]);			

			break;				


			case 3:                         //S5 Pressed
			SRbits.IPL = 3;	   // CPU priority set to 3

			//##########################################
			//  STEP 3:
			//  set the CPU interrupt priority level higher than Timer 5 
			//  and lower than Timer3
			//##########################################
			
			       // ###
			       // ###
			LCDClear();
			LCDHome();
			i=0; 
			while (MyString[i]!=0) LCDPut(MyString[i++]);			
						
			break;				


			case 4:

			//##########################################
			//  STEP 4:
			//  set the CPU interrupt priority level higher than both the Timers
			//##########################################
			
			             // ###
			LCDClear();
			LCDHome();
			i=0;
			while (MyString[i]!=0) LCDPut(MyString[i++]);			
			
			break;				

		}

		IEC0bits.T3IE = 1;	// Re-enable Timer 3 interrupt
		IEC1bits.T5IE = 1;	// Re-enable Timer 5 interrupt
	}

}



void ReadSwitch()
{
	Sw=0;
	while (Sw == 0)
	{
		Sw=1;
		if (PORTDbits.RD6 == 0)
		{
			Sw = 1;
			MyString[7]='1';
		}	
		if (PORTDbits.RD7 == 0)
		{
			Sw = 2;
			MyString[7]='2';
		}
		if (PORTAbits.RA7 == 0)
		{
			Sw = 3;
			MyString[7]='3';
		}
		if (PORTDbits.RD13 == 0)
		{
			Sw = 4;
			MyString[7]='4';
		}
	}
}

void TMR3Init()
{
	T3CON = 0x0030; 		// Internal Clock (FOSC/2), 1:256 prescale 

	PR3 = 0xF000;  			// 4.0 Second Time period.
	TMR3 = 0x0000;  

	IFS0bits.T3IF = 0;
	IEC0bits.T3IE = 1;		// Enable Timer 3 overflow interrupt

	T3CONbits.TON = 1;
}


void TMR5Init()
{
	T5CON = 0x0030; 		// Internal Clock (FOSC/2), 1:256 prescale, 16 bit Timer5
	
	PR5 = 0x7000;  			// 2 Second Time period.
	TMR5 = 0x0000;  

	IFS1bits.T5IF = 0;
	IEC1bits.T5IE = 1;		// Enable Timer 5 overflow interrupt

	T5CONbits.TON = 1;   // Timer 5 is off
}


void IOInit()
{
	AD1PCFG = 0xFFFF;
	LATAbits.LATA0 = 0;
	LATAbits.LATA1 = 0;
	TRISAbits.TRISA0 = 0;   // LED D3
	TRISAbits.TRISA1 = 0;   // LED D4
	TRISDbits.TRISD6 = 1;   // S3
	TRISDbits.TRISD13 = 1;  // S4
	TRISAbits.TRISA7 = 1;   // S5
	TRISDbits.TRISD7 = 1;   // S6
}


void Delay(int ticks)
{
	int i;

	for(i = 0; i < ticks; i++);
}

/**********************************************************************************/

void __attribute__((interrupt,no_auto_psv)) _T3Interrupt()
{
	long int i = 0;
    IFS0bits.T3IF = 0;  //Clear Timer3 interrupt flag bit 
    PORTAbits.RA0 = 1;		
	for(i = 0; i< 0xFFFFF ; i++) // Keep LED on and remain in interrupt for 3 sec
	{
	}			
    PORTAbits.RA0 = 0;
}




void __attribute__((interrupt,no_auto_psv)) _T5Interrupt()
{
	long int i = 0;

	IFS1bits.T5IF = 0;  //Clear Timer5 interrupt flag bit 

	PORTAbits.RA1 = 1;

	for(i = 0; i< 0x3000 ; i++)//  Keep LED on for .5 sec
	{
	}			

	PORTAbits.RA1 = 0;

}
