/*****************************************************************************
 *
 * ADC Demo for PIC24.
 *
 *****************************************************************************
 * FileName:        ADC.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Vidyadhar		  30/June/06	     ...	
 * Brant Ivey			7/3/07		Updated for v4.0 of class
 *****************************************************************************/

#include <p24FJ128GA010.h>

_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & ICS_PGx2 & FWDTEN_OFF)
_CONFIG2(IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_ON &  FNOSC_FRC & POSCMOD_NONE)


main()
{
	IOInit();
    ADCInit();              //Initialize the A/D converter
    ADCStart();				//Turn ADC on

	while(1)
	{

	  while(!IFS0bits.AD1IF);
	  IFS0bits.AD1IF = 0;	
	  
	  ADCAvg();				//Take average of 16 samples

	  __builtin_btg((unsigned int*)&LATB,2);	//Toggle pin for Oscope testing

	 } 
}


//To define ADCInit you may require some info.
// You will have to configure the registers:
// AD1CON1, AD1CON2, AD1CON3, AD1CHS, AD1PCFG, & AD1CSSL. 
// The description of these registers are given below.
ADCInit()
{

    //##########################################	
    //  STEP 1:
    // 	Select integer format result,
    // 	Select auto conversion start
    // 	Select Sample after conversion ends
    //##########################################	
    
	// YOUR CODE HERE *******          

    //##########################################	
    //  STEP 2:
	//  Select AVDD & AVSS as references,
	//  Disable Scan Mode for Ch0 input,
	//  Select interrupt after 16 sample/convert sequence,
	//  Select 16*1 buffer levels
	//  Always use MUX A
    //##########################################
    
	// YOUR CODE HERE *******          

    //##########################################	
    //  STEP 3:
	// 	Select AD Clock as derivative of system clock source,
    // 	Select 13TAD sampling time,
    // 	Select AD clcok frequency such that 16 samples are collected in 1 mSec
 	//	Assume FCY = 4 MHz or 1TCY = .25 uS  
    //##########################################
    
	// YOUR CODE HERE *******          

    //##########################################	
    //  STEP 4:
	//  Set the positive sample input channel for MUX A to use AN5
	//  Set the negative input channel for MUX A to use VR-
    //##########################################  
    
	// YOUR CODE HERE *******          

    //##########################################	
    //  STEP 5:
	// 	Set AD1PCFG so that the only pin using analog functionality is AN5
    //##########################################
    
	// YOUR CODE HERE *******          

    //##########################################	
    //  STEP 6:
	// 	Channel scanning is not enabled, so no input 
	//	channels should be selected for scanning
    //##########################################
    
	// YOUR CODE HERE *******          

}



IOInit()
{
	TRISDbits.TRISD6 = 1;			// Make PORTD<6> as input.
	TRISA = 0xFF00;					// Make PORTA<0:7> as outputs, PORTA<8:15> as inputs.
	TRISBbits.TRISB2 = 0;			// Make PORTB<2> output for pin toggling
}

ADCStart()
{
	AD1CON1bits.ADON = 1;     // Turn on the A/D converter
}

ADCAvg()
{

unsigned long int temp;

temp = 	ADC1BUF0 + ADC1BUF1 + ADC1BUF2 + ADC1BUF3 + 
		ADC1BUF4 + ADC1BUF5 + ADC1BUF6 + ADC1BUF7 + 
		ADC1BUF8 + ADC1BUF9 + ADC1BUFA + ADC1BUFB + 
		ADC1BUFC + ADC1BUFD + ADC1BUFE + ADC1BUFF;
LATA = temp/64;     // 64 = 256/(16*1024)  take average, scale 10-bits to 8-bits.

}
